package main

import (
	"fmt"
	"io/ioutil"
	"log"
	"os"
	"path"
	"time"

	"git.wow.st/gmp/ble"

	"gopkg.in/yaml.v2"

	"gioui.org/app"
	"gioui.org/io/system"
	"gioui.org/layout"
	"gioui.org/text"
	"gioui.org/unit"
	"gioui.org/widget"
	"gioui.org/widget/material"

	_ "gioui.org/font/gofont"
)

type conf struct {
}

var Config conf

func main() {
	conffile := path.Join(confDir, "config.yml")
	if _, err := os.Stat(conffile); os.IsNotExist(err) {
		fd, err := os.Create(conffile)
		if err != nil {
			log.Fatal("Cannot create configuration file: ", err)
		}
		fd.Close()
	}

	confbytes, err := ioutil.ReadFile(conffile)
	if err != nil {
		log.Fatal("Cannot read configuration file: ", err)
	}
	if err = yaml.UnmarshalStrict(confbytes, &Config); err != nil {
		log.Fatal("Cannot parse configuration file: ", err)
	}
	go eventloop()
	app.Main()
}

func eventloop() {
	w := app.NewWindow(
		app.Size(unit.Dp(400), unit.Dp(400)),
		app.Title("HRM"),
	)
	th := material.NewTheme()
	gtx := &layout.Context{Queue: w.Queue()}

	sysinset := &layout.Inset{}
	resetSysinset := func(x system.Insets) {
		sysinset.Top = x.Top
		sysinset.Bottom = x.Bottom
		sysinset.Left = x.Left
		sysinset.Right = x.Right
	}
	margin := layout.UniformInset(unit.Dp(10))

	b := ble.NewBLE()

	state := "starting"
	var periph ble.Peripheral
	periphs := make([]ble.Peripheral, 0)
	btns := make([]*widget.Button, 0)

	var autoconnectID string // should load from config file
	autoconnectID = "93D3A64F-1664-497D-8B01-77951DB8E0F3"

	var page, offpage, scanpage, connpage, hrpage func()

	f := &layout.Flex{Axis: layout.Vertical}
	offpage = func() {
		c1 := f.Rigid(gtx, func() {
			th.Body1("Heart Rate Monitor").Layout(gtx)
		})
		c2 := f.Rigid(gtx, func() {
			th.Body1("Bluetooth is Powered Off").Layout(gtx)
		})
		f.Layout(gtx, c1, c2)
	}
	scanpage = func() {
		c1 := f.Rigid(gtx, func() {
			th.Body1("Heart Rate Monitor").Layout(gtx)
		})
		c2 := f.Rigid(gtx, func() {
			th.Body1(state).Layout(gtx)
		})
		c3 := f.Rigid(gtx, func() {
			lst := &layout.List{Axis: layout.Vertical}
			lst.Layout(gtx, len(periphs), func(i int) {
				th.Button(periphs[i].Name).Layout(gtx, btns[i])
				if btns[i].Clicked(gtx) {
					b.StopScan()
					b.Connect(periphs[i])
					page = connpage
					w.Invalidate()
				}
			})
		})
		f.Layout(gtx, c1, c2, c3)
	}

	connpage = func() {
		c1 := f.Rigid(gtx, func() {
			th.Body1("Heart Rate Monitor").Layout(gtx)
		})
		c2 := f.Rigid(gtx, func() {
			th.Body1("Connecting").Layout(gtx)
		})
		c3 := f.Rigid(gtx, func() {
			th.Body1(periph.Name).Layout(gtx)
		})
		f.Layout(gtx, c1, c2, c3)
	}

	hrpage = func() {
		c1 := f.Rigid(gtx, func() {
			th.Body1("Heart Rate Monitor").Layout(gtx)
		})
		c2 := f.Rigid(gtx, func() {
			th.Body1(periph.Name).Layout(gtx)
		})
		c3 := f.Rigid(gtx, func() {
			l := th.H1(fmt.Sprintf("%d", b.HR()))
			l.Alignment = text.Middle
			l.Layout(gtx)
		})
		f.Layout(gtx, c1, c2, c3)
	}

	page = offpage

	tick := time.NewTicker(time.Second / 30)
	events := b.Events()

	for {
		select {
		case e := <-events:
			switch e := e.(type) {
			case ble.UpdateStateEvent:
				state = e.State
				if state != "powered on" {
					page = offpage
				} else {

					if autoconnectID != "" {
						b.Connect(ble.Peripheral{Identifier: autoconnectID})
						page = connpage
					} else {
						page = scanpage
						b.Scan()
					}
				}
				w.Invalidate()
			case ble.DiscoverEvent:
				fmt.Printf("found %s\n", e.Peripheral.Identifier)
				periphs = append(periphs, e.Peripheral)
				btns = append(btns, &widget.Button{})
				if e.Peripheral.Identifier == autoconnectID {
					b.Connect(e.Peripheral)
				}
			case ble.ConnectEvent:
				fmt.Printf("Connect event\n")
				state = "connected"
				periph = e.Peripheral
				ble.DiscoverServices(e.Peripheral)
				page = hrpage
				w.Invalidate()
			case ble.ConnectTimeoutEvent:
				fmt.Printf("Connect timeout\n")
				state = "timeout"
				autoconnectID = ""
				page = scanpage
				b.Scan()
				w.Invalidate()
			}
		case <-tick.C:
			w.Invalidate()
		case e := <-w.Events():
			switch e := e.(type) {
			case system.DestroyEvent:
				return
			case system.FrameEvent:
				gtx.Reset(e.Config, e.Size)
				resetSysinset(e.Insets)
				sysinset.Layout(gtx, func() {
					margin.Layout(gtx, page)
				})
				e.Frame(gtx.Ops)
			}
		}
	}
}
